#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.association.interfaces import IAssociationItem
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IFormContextPermissionChecker
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent

# import packages
from persistent import Persistent
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty


@implementer(IAssociationItem)
class AssociationItem(Persistent, Contained):
    """Base association item persistent class"""

    icon_class = ''
    icon_hint = ''

    visible = FieldProperty(IAssociationItem['visible'])

    def get_url(self, request=None, view_name=None):
        return absolute_url(self, request, view_name)


@adapter_config(context=IAssociationItem, provides=IFormContextPermissionChecker)
class AssociationItemPermissionChecker(ContextAdapter):
    """Association item permission checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, IWfSharedContent)
        return IFormContextPermissionChecker(content).edit_permission


@subscriber(IObjectAddedEvent, context_selector=IAssociationItem)
def handle_added_association(event):
    """Handle added association item"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectModifiedEvent, context_selector=IAssociationItem)
def handle_modified_association(event):
    """Handle modified association item"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectRemovedEvent, context_selector=IAssociationItem)
def handle_removed_association(event):
    """Handle removed association item"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))
