#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.shared.common.interfaces import ISharedSite
from pyams_i18n.interfaces import II18n

# import packages
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.registry import get_utilities_for
from pyams_utils.request import query_request
from pyams_utils.vocabulary import vocabulary_config
from zope.interface import implementer
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary


@implementer(ISharedSite)
class SharedSite(BTreeOrderedContainer):
    """Shared site persistent class"""


@vocabulary_config(name='PyAMS shared sites')
class SharedSiteVocabulary(SimpleVocabulary):
    """Shared sites vocabulary"""

    interface = ISharedSite

    def __init__(self, context):
        request = query_request()
        super(SharedSiteVocabulary, self).__init__([SimpleTerm(v, title=II18n(t).query_attribute('title',
                                                                                                 request=request))
                                                    for v, t in get_utilities_for(self.interface)])
